// Library Reference for Motor Shield:
//    https://learn.adafruit.com/adafruit-motor-shield-v2-for-arduino/library-reference
// Motor Shield Overview:
//    https://learn.adafruit.com/adafruit-motor-shield-v2-for-arduino/overview

// To include the Motor Shield library,
// 1. Open "Sketch>Include Library>Manage Libraries..."
// 2. Search for Adafruit Motor Shield v2 Library and add it
// Also, there's more example sketches in:
//    "File>Examples>Adafruit Motor Shield V2 Library"

// MOTOR PARTY:
// A /distance sensor/ that checks whether you're close enough to have a party
// A /stepper motor /  that spins in a different direction every time you start a party
// A /dc motor/ that spins around for an extra 2 seconds after the party is over
// A /big servo/ that spins 180 degrees, and then starts back at the beginning
// A /little servo/ that spins according to value from the distance sensor

#include <Wire.h>
#include <Servo.h>
#include <Adafruit_MotorShield.h>

#define distanceSensorPin A0

//------------------------------------------------------
// Is a party happening currently?
bool bPartyHappening = false;

// Expected extrema for sensor values
int maxRawSensorValue = 650; // values more than this will be ignored
int minRawSensorValue =  50; // values less than this will be ignored

// Timestamp when we were most recently partying
unsigned long mostRecentPartyTime;

// For how long should we spin the DC motor (in milliseconds)
// after the last party has concluded
int motorSpinTime = 2000;

// what degree the servo is rotated to
float servo2Degree = 0;

// what direction the stepper is spinning, either -1 or +1
int stepperStep = -1;

// running average to smooth distance servo
float runningAvgServo = 90.0;

//------------------------------------------------------
// create the motor shield object
Adafruit_MotorShield AFMS = Adafruit_MotorShield();

// connect a stepper motor to motor port #2 (M3 and M4) with 200 steps per revolution (3.6 degrees)
Adafruit_StepperMotor *stepper = AFMS.getStepper(6000, 2);

// connect a DC motor to port M1
Adafruit_DCMotor *motor = AFMS.getMotor(1);

// connect a servo to servo 1 and servo 2
Servo servo1;
Servo servo2;

//------------------------------------------------------
void setup() {
  // Start the serial port.
  // We need this in order to print debugging info.
  Serial.begin(9600);

  // Initialize the AdaFruit Motor Shield
  AFMS.begin();

  // attach servo1 to pin #10
  servo1.attach(10);

  // attach servo2 to pin #9
  servo2.attach(9);

  // setup motor M1
  motor->setSpeed(80);
  motor->run(RELEASE);

  // set up the stepper
  stepper->setSpeed(10);
}


//---------------------------------------------------------
void loop() {

  // Update our world (sensors & motors) every 3 milliseconds
  delay(3);

  // Fetch  values from the IR distance sensor.
  int rawSensorValue = analogRead(distanceSensorPin);
  Serial.println(rawSensorValue);

  // If the sensor value is within an acceptable range
  if ((rawSensorValue < maxRawSensorValue) &&
      (rawSensorValue > minRawSensorValue)) {

    // Store the current time. 
    // It's the most recent interaction timestamp.
    mostRecentPartyTime = millis();

    // If we've *just* entered the acceptable distance zone,
    // flip the stepper direction, and update our bookkeeping.
    if (bPartyHappening == false) {
      Serial.println("Start the party!!!!");
      stepperStep = stepperStep * -1;
      bPartyHappening = true;
    }

    // Spin the stepper 5 little turns at a time.
    if (stepperStep == 1) {
      stepper->step(5, FORWARD, INTERLEAVE);
    } else {
      stepper->step(5, BACKWARD, INTERLEAVE);
    }

    // Set servo #1 proportional to the sensor distance.
    float newServoAngle = map(rawSensorValue, minRawSensorValue, maxRawSensorValue,  0.0, 180.0);
    runningAvgServo = (0.9 * runningAvgServo) + (0.1 * newServoAngle);
    servo1.write (runningAvgServo);

    // Spin servo #2: forward, then quickly back
    servo2Degree = servo2Degree + 2;
    if (servo2Degree > 180) {
      servo2Degree = 0;
    }
    servo2.write(servo2Degree);

  } else {
    // Stop the motor
    motor->run(RELEASE);
    bPartyHappening = false;
  }

  // Run the motor if we're partying, or if we're within the extra time
  if ((millis() - mostRecentPartyTime) < motorSpinTime) {
    motor->run(FORWARD);
  }
}
